/**
 * @file
 */
#ifdef BACKEND_CUDA
#include "numbirch/cuda/transform.inl"
#endif
#ifdef BACKEND_EIGEN
#include "numbirch/eigen/transform.inl"
#endif
#include "numbirch/common/transform.inl"

#define TERNARY(f, R) \
    TERNARY_FIRST(f, R, real) \
    TERNARY_FIRST(f, R, int) \
    TERNARY_FIRST(f, R, bool)
#define TERNARY_FIRST(f, R, T) \
    TERNARY_SECOND(f, R, T, real) \
    TERNARY_SECOND(f, R, T, int) \
    TERNARY_SECOND(f, R, T, bool)
#define TERNARY_SECOND(f, R, T, U) \
    TERNARY_THIRD(f, R, T, U, real) \
    TERNARY_THIRD(f, R, T, U, int) \
    TERNARY_THIRD(f, R, T, U, bool)
#define TERNARY_THIRD(f, R, T, U, V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 2), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 0), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), U, NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), U, V) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 0), V) \
    TERNARY_SIG(f, R, T, U, NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, T, U, V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 2), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 2), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), U, NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), NUMBIRCH_ARRAY(U, 0), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), U, NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 2), U, V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 2), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 2), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 2), V) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 2), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 2), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), U, NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, T, U, NUMBIRCH_ARRAY(V, 2)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 1), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), U, NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), NUMBIRCH_ARRAY(U, 0), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), U, NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 1), U, V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 1), V) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 1), NUMBIRCH_ARRAY(V, 0)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 1), V) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, NUMBIRCH_ARRAY(T, 0), U, NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, T, NUMBIRCH_ARRAY(U, 0), NUMBIRCH_ARRAY(V, 1)) \
    TERNARY_SIG(f, R, T, U, NUMBIRCH_ARRAY(V, 1))
#define TERNARY_SIG(f, R, T, U, V) \
    template R<T,U,V> f<T,U,V,int>(const T&, const U&, const V&);

namespace numbirch {
TERNARY(ibeta, real_t)
TERNARY(where, implicit_t)
}
